@@ -86,7 +86,7 @@ GtkSourceStyle          *_gtk_source_style_scheme_get_draw_spaces_style
 								(GtkSourceStyleScheme *scheme);
 gboolean		 _gtk_source_style_scheme_get_current_line_color
 								(GtkSourceStyleScheme *scheme,
-								 GdkColor             *color);
+								 GdkRGBA              *color);
 G_END_DECLS
 
 #endif  /* __GTK_SOURCE_STYLE_SCHEME_H__ */
